% Dragomir Yankov
% the same as runLibSvm but calls matlabs quadprog solver instead
function Params = runMatSvm(X, nu, gamma, weights, mfacenters, plotting)
    Lbl = zeros(size(X,1),1);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % running one class svm and clustering the dense regions, 
    % i.e. running SVC
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    % detect the best gamma.
    model = svmmattrain(X, nu, gamma, weights)
    svs = full(model.SVs);   % the detected support vectors
    sv_inds = getIndices(X, svs); % the indices of the corresponding elements
    X1 = X;
    for i = 1:size(mfacenters,1)
        X1 = [X1; mfacenters(i,:)];  % add the mean of the mfa clusters, so that their label could also be detected
    end;
    % C++ libsvm code
    %[train_label, accuracy, prob_estimates] = svmpredict(ones(size(X,1),1), X, model);
    % my java predict code
    train_label = svmpredict1(X1, model, gamma);
    if (size(X,2) == 2 && plotting)
        plotContours(X, model, gamma);
    end;
    Lbls = clusterData(X1, model, sv_inds, train_label);        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % plot the detected clusters and the (bounded) support vectors
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Params = cell(4,1);
    Params{1} = model;
    Params{2} = train_label;
    Params{3} = Lbls(1:size(X,1),1); % labels for the data points
    Params{4} = Lbls(size(X,1)+1:size(Lbls,1),1); % labels for the means of the mfa clusters
    %plotClustering(X, model, train_label, Lbls);
    %plotContours(X, model, gamma);
    
function model = svmmattrain(X, nu, gamma, weights)    
    n = size(X,1);
    K = kernelMatrix(X, gamma);  %K_ij = <x_i,x_j>
    f = zeros(1,n);
    A = eye(n);
    if (size(weights,1) ~= n)
        b= (ones(n,1)/(n*nu));
    else 
        b = (ones(n,1)./weights)/(n*nu);
    end;
    Aeq = ones(1,n);
    beq = 1;
    lb = zeros(n,1);
    ub = b;
    oldopts = optimset;
    options = optimset(oldopts, 'MaxIter', 1000);
    [alpha,fval,exitflag] = quadprog(K,f,A,b,Aeq,beq,lb,ub,[],options);
    exitflag_ = exitflag
    % fix precision issues
    inds = find((alpha > -0.000000001) & (alpha < 0.000000001));
    alpha(inds, 1) = 0;
    % end fix
    
    model = struct('Parameters', [2;2;3;gamma;0], ...
                   'nr_class', 2, ...
                   'totalSV',0,...
                   'rho',0,...
                   'Label',[], ...
                   'ProbA',[], ...
                   'ProbB',[], ...
                   'nSV', [],...
                   'sv_coef',[],...
                   'SVs', []);    
    svs_inds = find(alpha>0);
    model.totalSV = size(svs_inds,1);
    model.sv_coef = alpha(svs_inds,1);    
    model.SVs = X(svs_inds,:);
    model.SVs = sparse(model.SVs);
    X_i = KernelG(X(svs_inds(1,1),:), model);
    model.rho = model.sv_coef'*X_i;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% mimics the svmpredict from the dll but rather than -1 1 returns real 
% value labels corresponding to the margine
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function L = svmpredict1(X, model, gamma) 
    m = size(X,1);
    L = zeros(m,1);
    ro = model.rho;
    for i = 1:m
        L(i,1) = model.sv_coef'*KernelG(X(i,:), model) - ro;    
    end;
    inds = find((L > -0.000000001) & (L < 0.000000001));
    L(inds, 1) = 0;

function L = clusterData(X, model, sv_inds, train_label)
    A = buildAdjacency(X, model, sv_inds, train_label);
    connComp = manifolds.ConnectedComponents;
    L = connComp.clabel(A);

% check the line between each example and the support vectors. 
% If for all the points on the line the radius is smaller than the radius of the
% bounding SV sphere then the point is in one cluster with this support vector
function A = buildAdjacency(X, model, sv_inds, train_label)    
    m = size(X,1);
    predictor = manifolds.SVMPredict_java
    A = predictor.bldAdjacency(X, train_label, model.sv_coef, sv_inds, model.SVs, model.rho, model.Parameters(4,1));
    
function A = buildAdjacency1(X, model, sv_inds, train_label)    
    m = size(X,1);
    n = size(sv_inds,1);
    A = zeros(m);
    for i = 1:m
        if (train_label(i,1) < 0) % bounded support vector (BSV)
            continue;
        end;
        for j = 1:n %check the line connecting example i to all support vectors j
            svind = sv_inds(j,1);
            if (svind == i)
                A(i,i) = 1;
                continue;
            end;
            %if (A(i,svind) == 1)
            if (A(svind,i) == 1)
                continue;
            end;
            X_new = pointsOnLine(X(i,:), X(svind,:), 20);
            [pts_label, accuracy, prob_estimates] = mex_svmpredict(ones(size(X_new,1),1), X_new, model);
            %pts_label = svmpredict1(X_new, model, model.Parameters(4,1))
            if (size(find(pts_label < 0),1) == 0)
                A(i,svind) = 1;
                A(svind,i) = 1;
            end;
        end;
    end;    
    
function L = svmmatpredict(Lbl, X, model)
    %K = X model.SVs
    L = [];
    
function X_new = pointsOnLine(x1, x2, numpoints)
    step = (x2 - x1)/(numpoints+1);
    X_new = [];
    for i = 1:numpoints
        X_new = [X_new; x1 + i*step];
    end;    
    
function Ind = getIndices(X, svs)
    Ind = [];
    for i = 1:size(svs,1)
        tmpInd = findInd(X, svs(i,:));
        Ind = [Ind; tmpInd];
    end;
    
function Ind = findInd(X, sv)
    for i = 1:size(X,1)
        isInd = 1;
        for j = 1:size(X,2)
            if (X(i,j) ~= sv(1,j))
                isInd = 0;
                break;
            end;
        end;
        if (isInd == 1)
            Ind = i;
            return;
        end;
    end;
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% computes the vector K(x, x_i) i = 1,N
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function K_xxi = KernelG(ptX, model)
    SVs = model.SVs;
    m = size(SVs,1);
    Xi = repmat(ptX, m, 1);
    normXX = sum((Xi - SVs).*(Xi-SVs),2);
    %normXX = sqrt(sum((Xi - SVs).*(Xi-SVs),2));
    K = exp(ones(m,1));
    gamma = model.Parameters(4,1);
    K_xxi = K.^(-gamma*normXX);    
    
function K = kernelMatrix(X, gamma)
    m = size(X,1);
    K = exp(ones(m));
    normXX = [];
    for i = 1:m
        Xj = repmat(X(i,:), m, 1);
        normXX = [normXX sum((X - Xj).*(X-Xj),2)];
        %normXX = [normXX sqrt(sum((X - Xj).*(X-Xj),2))];
    end;
    K = K.^(-gamma*normXX);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [X, Y] = loadDataXY(file)    
    dims_data = 2;
    dims_file = 3;
    D = loadData(file,dims_file);
    cols= size(D,2);
    X = D(:,1:cols-1);
    Y = D(:,cols);
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
% Plotting functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plotOrigData(X, Y)    
    inds = find(Y == 1);
    figure;
    plot(X(inds,1), X(inds,2), '.b');
    inds = find(Y==2);
    hold on;
    plot(X(inds,1), X(inds, 2), '.r');   
    title('Test data');
    %axis([-4,4,-4,4]);    

function plotClustering(X, model, train_label, Lbls)
    svs = full(model.SVs);
    sv_inds = getIndices(X, svs);
    inds_neg = find(train_label < 0);
    inds_pos = find(train_label > 1);
    %figure;
    %hold on;
%     plot(X(inds_neg,1), X(inds_neg,2), '.k');
     plot(svs(:,1), svs(:,2), 'ok'); 
%    plot3(X(inds_neg,1), X(inds_neg,2), -0.15*ones(size(inds_neg,1),1), '.k');
%    plot3(svs(:,1), svs(:,2), -0.15*ones(size(svs,1),1), 'ok'); 

    lbls_type = unique(Lbls);
    num_lbls = size(lbls_type,1);
    color_map = ['b'; 'r'; 'g'; 'm'; 'y'; 'c']; 
    for i = 1:num_lbls
        inds = find(Lbls(:,1) == lbls_type(i,1));
        if (lbls_type(i,1) == 0) % plot the BSVs
            scol = '.k';
        else 
            col = rem(i-1, size(color_map,1))+1;
            scol = ['.' color_map(col,1)];
        end;
        plot(X(inds,1),X(inds,2), scol, 'LineWidth', 2);        
        %plot3(X(inds,1),X(inds,2), -0.15*ones(size(inds,1),1), ['.' color_map(col,1)], 'LineWidth', 1);
    end;
    %axis([-4,4,-4,4]);    

function C = getColormap()
    X = [0.6 0.2 0.39]
    Y = [0.601 0.601 1]
    Z = [0.746 0.746 0.601];
    step1 = (Y-X)/3;
    step2 = (Z-Y)/3;
    C = [X;X+step1;X+2*step1;Y;Y+step2;Y+2*step2;Z];    
    
function plotContours(X, model, gamma)
    minx = min(X(:,1));
    maxx = max(X(:,1));
    miny = min(X(:,2));
    maxy = max(X(:,2));
    stepx = (maxx - minx)/50;
    stepy = (maxy - miny)/50;
    Xcoord = minx:stepx:maxx;
    Ycoord = miny:stepy:maxy;
    Z = zeros(size(Ycoord,2), size(Xcoord,2));
    for i = 1:size(Ycoord,2)
        for j = 1:size(Xcoord,2)
            Z(i,j) = model.sv_coef'*KernelG([Xcoord(1,j) Ycoord(1,i)], model) - model.rho;    
            if (Z(i,j) < -0.05)
                Z(i,j) = -0.05;
            elseif (Z(i,j) > 1)
                Z(i,j) = 1;
            end;
        end;
    end;
     figure;
     [Xm,Ym] = meshgrid(Xcoord,Ycoord);
    [c, h]=contour3(Xm,Ym,Z,[0 0.1 0.2 0.3]);
    set(h, 'LineWidth', 3)
    hold on;
    surface(Xm,Ym,Z,'EdgeColor',[.8 .8 .8],'FaceColor','none')        
    contour3(Xm,Ym,Z,[0 0.1 0.2 0.3]); 
    %C = getColormap;
    colormap cool    
    %colormap(C);
    grid off
    view(35,65)
    %axis tight
    colormap cool

    figure;
    %Z = Z*100;
    surfcc(Xm,Ym,Z)
    colormap cool
    %colormap(C);
    view(55,65)
    hold on;
    
    figure;
    [c, h] = contour(Xm, Ym, Z, [0 0]);
    %colormap([0.746 0.746 0.601])
    %colormap([129/256 125/255 1])
    colormap([192/256 192/256 192/256])
    %colormap([1 0 0]);
    set(h, 'LineWidth', 3);
    %set(h, 'MarkerEdgeColor', [0.746 0.746 0.601]);
    hold on;
    svs = model.SVs;
    %plot(svs(:,1),svs(:,2), 'ok', 'LineWidth', 2);        
    k = 1;
