%
% Finds out what true labeling the cluster labeling corresponds to.
% For the best possible labeling measure the percentage of correctly
% labeled samples
function [quality, labeling, permut] = testClusterQuality(k_clust, trueLabels, clusterLabels)
    % find all possible permutations for the clusters
    permLabels = perms(1:k_clust);
    scores = [];
    allPermutations = [];
    for i = 1:size(permLabels,1)
        tmpL = permLabels(i,:)
        permClustLabels = permuteLabels(permLabels(i, :), clusterLabels);
        allPermutations = [allPermutations permClustLabels];
        scores = [scores getScore(trueLabels, permClustLabels)];
    end;
    [quality lblInd] = max(scores)
    labeling = allPermutations(:, lblInd);
    permut = permLabels(lblInd, :);
    
    
function L = permuteLabels(permLabels, clustLabels)
    L = zeros(size(clustLabels,1), 1);
    for i = 1:size(clustLabels,1)
        L(i,1) = permLabels(clustLabels(i,1));
    end;
    
%
% returns the percentage of correctly clustered according to that labeling
function S = getScore(data, labels)
    correct = 0;
    all = 0;
    for i = 1:size(data,1)
        if (data(i,1) == labels(i,1))
            correct = correct + 1;
        end;
    end
    S = correct / size(data,1);
    
    
    